@@ -1,4 +1,4 @@
-from .core import KoshLoader, KoshFileLoader  # noqa
+from .core import KoshLoader, KoshFileLoader, get_graph  # noqa
 from .jsons import JSONLoader  # noqa
 try:
     from .pil import PILLoader  # noqa
@@ -13,3 +13,7 @@ try:
     from .UltraLoader import UltraLoader  # noqa
 except ImportError:
     pass  # we do not have pydv
+try:
+    from .sidre import SidreMeshBlueprintFieldLoader  # noqa
+except ImportError:
+    pass  # we do not have conduit
