@@ -1,7 +1,13 @@
-from .core import KoshTransformer, get_path, kosh_cache_dir  # noqa
+from .core import KoshTransformer  # noqa
 from .npy import KoshSimpleNpCache  # noqa
 try:
     from .skl import StandardScaler, KMeans, DBSCAN, Splitter  # noqa
 except NameError:
     # no skl...
     pass
+try:
+    from .sidre import SidreFeatureMetrics  # noqa
+except NameError:
+    # no conduit
+    pass
+from kosh import kosh_cache_dir  # noqa
\ No newline at end of file
